package	jp.co.sds_corp.hibiya.monitor;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.border.TitledBorder;

import jp.co.sds_corp.hibiya.action.IAction;
import jp.co.sds_corp.hibiya.layout.ILayout;
import jp.co.sds_corp.hibiya.monitor.action.IActionPanel;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* {@link IAction}を設定するダイアログです。
*/
public class ActionDialog extends JDialog
{
	private static final long serialVersionUID = 1L;
	SetupParam	setup;
	JPanel		buttonPnl = new JPanel();
	JButton		BTN_OK = new JButton( "OK" );
	JButton		BTN_CANCEL = new JButton( "Cancel" );
	IActionPanel actionPanel;
	LayoutPanel	layoutPanel;
	Box			layoutBox;
	boolean		bOk;
	ResourceBundle	res;

	IAction		action;

	public ActionDialog( Frame parent, SetupParam setup ) {
		super( parent );
		this.setup = setup;
		layoutPanel = new LayoutPanel( parent, setup );

		createControls();
	}

	public ActionDialog( Dialog parent, SetupParam setup ) {
		super( parent );
		this.setup = setup;
		layoutPanel = new LayoutPanel( parent, setup );

		createControls();
	}

	protected void createControls() {
		res = ResourceBundle.getBundle( "resource.monitor" );
		String text;
		setModal( true );
		setResizable( false );
		text = res.getString( "action.title" );
		setTitle( text );
		setSize( 477, 421 );
		setFont( setup.getDefaultFont() );
		getContentPane().setLayout( new BorderLayout() );

		WindowUtility.setHideOnEscape( this );

		layoutBox = new Box( BoxLayout.X_AXIS );
		text = res.getString( "action.format.title" );
		layoutBox.setBorder( new TitledBorder( text ));
		layoutBox.add( layoutPanel );

		buttonPnl.add( BTN_OK );
		buttonPnl.add( BTN_CANCEL );

		BTN_OK.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onOk();
			}
		});
		BTN_CANCEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onCancel();
			}
		});

	}

	public IAction getAction() {
		return action;
	}

	public void setAction( IAction action ) {
		this.action = action;
	}

	public void clearAction() {
		this.action = null;
	}

	public void setVisible( boolean var ) {
		if ( var ) {
			bOk = false;
			getContentPane().removeAll();
			try  {
				JPanel panel = new JPanel();
				panel.setLayout( new BoxLayout( panel, BoxLayout.Y_AXIS ));
				actionPanel = setup.getActionPanel( action );
				if ( actionPanel == null ) {
					String msg = res.getString( "action.err1.message" );
					String title = res.getString( "action.err1.title" );
					JOptionPane.showMessageDialog( this, msg, title,
						JOptionPane.ERROR_MESSAGE );
					return;
				}
				actionPanel.setAction( action );
				actionPanel.setEnabled( isEnabled() );
				panel.add( (JPanel )actionPanel );
				try {
					layoutPanel.setEventLayout( action.getLayout() );
					panel.add( layoutBox );
				} catch ( UnsupportedOperationException uex ) {
				}

				getContentPane().add( panel, BorderLayout.NORTH );
				getContentPane().add( buttonPnl, BorderLayout.SOUTH );
				BTN_OK.requestFocus();
				getRootPane().setDefaultButton( BTN_OK );
				SwingUtilities.updateComponentTreeUI( this );
			} catch ( Exception ex ) {
				CmnLog.error( this, "error", ex );
				String msg = res.getString( "action.err2.message" );
				String title = res.getString( "action.err2.title" );
				JOptionPane.showMessageDialog( this,
						msg + "\n" + ex, title, JOptionPane.ERROR_MESSAGE );
			}
		}
		super.setVisible( var );
	}

	public void onOk() {
		try {
			actionPanel.updateAction();
		} catch ( Exception ex ) {
			CmnLog.error( this, "error" + ex );
			String title = res.getString( "action.err3.title" );
			String msg = res.getString( "action.err3.message" );
			if ( ex.getMessage() == null ) {
				msg += "\n" + ex.toString();
			} else {
				msg += "\n" + ex.getMessage();
			}
			JOptionPane.showMessageDialog( this,
					msg, title, JOptionPane.ERROR_MESSAGE );
			return;
		}
		layoutPanel.updateLayout();
		try {
			ILayout layout = layoutPanel.getEventLayout();
			action.setLayout( layout );
		} catch ( UnsupportedOperationException uex ) {
		}
		bOk = true;
		setVisible( false );
	}

	public void onCancel() {
		setVisible( false );
	}

	public boolean isOk() {
		return bOk;
	}

	public void setEnabled( boolean bEn ) {
		if ( !bEn ) {
			BTN_CANCEL.requestFocus();
		}
		super.setEnabled( bEn );
		BTN_OK.setEnabled( bEn );
		if ( actionPanel != null ) {
			actionPanel.setEnabled( bEn );
		}
		layoutPanel.setEnabled( bEn );
	}

}
