package	jp.co.sds_corp.hibiya.monitor;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import jp.co.sds_corp.hibiya.connector.IConnector;
import jp.co.sds_corp.hibiya.monitor.connector.IConnectorPanel;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* {@link IConnector}を設定するダイアログです。
*/
public class ConnectorDialog extends JDialog
{
	private static final long serialVersionUID = 1L;
	SetupParam	setup;
	JPanel		buttonPnl = new JPanel();
	JButton		BTN_OK = new JButton( "OK" );
	JButton		BTN_CANCEL = new JButton( "Cancel" );
	IConnectorPanel connectorPanel;
	boolean		bOk;
	ResourceBundle	res;

	IConnector		connector;

	public ConnectorDialog( Frame parent, SetupParam setup ) {
		super( parent );
		this.setup = setup;

		createControls();
	}

	public ConnectorDialog( Dialog parent, SetupParam setup ) {
		super( parent );
		this.setup = setup;

		createControls();
	}

	protected void createControls() {
		res = ResourceBundle.getBundle( "resource.monitor" );
		String text;
		setModal( true );
		setResizable( false );
		text = res.getString( "connector.title" );
		setTitle( text );
		setSize( 477, 421 );
		setFont( setup.getDefaultFont() );
		getContentPane().setLayout( new BorderLayout() );

		WindowUtility.setHideOnEscape( this );

		buttonPnl.add( BTN_OK );
		buttonPnl.add( BTN_CANCEL );

		BTN_OK.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onOk();
			}
		});
		BTN_CANCEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onCancel();
			}
		});

	}

	public IConnector getConnector() {
		return connector;
	}

	public void setConnector( IConnector connector ) {
		this.connector = connector;
	}

	public void clearConnector() {
		this.connector = null;
	}

	public void add() {
		IConnector con = createConnector();
		if ( con == null ) {
			return;
		}
		setConnector( con );
		setVisible( true );
	}

	private IConnector createConnector() {
		Map<String,SetupParam.Registry> mapCon = setup.getConnectorMap();
		if ( mapCon.size() == 0 ) {
			return null;
		}
		String[] options = new String[ mapCon.size() ];
		int nPos = 0;
		HashMap<String,SetupParam.Registry> mapSel = new HashMap<String,SetupParam.Registry>();
		Iterator<String> itr = mapCon.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			SetupParam.Registry reg = (SetupParam.Registry )mapCon.get( key );     
			mapSel.put( reg.getType(), reg );
			options[nPos++] = reg.getType();
		}
		String title = res.getString( "connector.add.title" );
		String msg = res.getString( "connector.add.message" );
		String sel = (String )JOptionPane.showInputDialog( getParent(),
			msg, title, JOptionPane.INFORMATION_MESSAGE,
			null, options, options[0] );
		if ( sel == null ) {
			return null;
		}

		String cls = ((SetupParam.Registry )mapSel.get( sel )).getOrgClass();
		IConnector ret = null;
		try {
			ret = (IConnector )Class.forName( cls ).newInstance();
		} catch ( Exception ex ) {
			CmnLog.error( this, "error", ex );
			title = res.getString( "connector.err3.title" );
			msg = res.getString( "connector.err3.message" );
			JOptionPane.showMessageDialog( this,
				msg + "\n" + ex + "\n" + cls,
				title, JOptionPane.ERROR_MESSAGE );
			return null;
		}
		return ret;
	}

	public void setVisible( boolean var ) {
		if ( var ) {
			bOk = false;
			getContentPane().removeAll();
			try  {
				JPanel panel = new JPanel();
				panel.setLayout( new BoxLayout( panel, BoxLayout.Y_AXIS ));
				connectorPanel = setup.getConnectorPanel( connector );
				if ( connectorPanel == null ) {
					String msg = res.getString( "action.err1.message" );
					String title = res.getString( "connector.err1.title" );
					JOptionPane.showMessageDialog( this, msg, title,
						JOptionPane.ERROR_MESSAGE );
					return;
				}
				connectorPanel.clear();
				connectorPanel.setConnector( connector );
				connectorPanel.setEnabled( isEnabled() );
				panel.add( (JPanel )connectorPanel );

				getContentPane().add( panel, BorderLayout.NORTH );
				getContentPane().add( buttonPnl, BorderLayout.SOUTH );
				BTN_OK.requestFocus();
				getRootPane().setDefaultButton( BTN_OK );
				SwingUtilities.updateComponentTreeUI( this );
			} catch ( Exception ex ) {
				CmnLog.error( this, "error", ex );
				String msg = res.getString( "connector.err2.message" );
				String title = res.getString( "connector.err2.title" );
				JOptionPane.showMessageDialog( this,
						msg + "\n" + ex, title, JOptionPane.ERROR_MESSAGE );
			}
		}
		super.setVisible( var );
	}

	public void onOk() {
		try {
			connectorPanel.updateConnector();
		} catch ( Exception ex ) {
			CmnLog.error( this, "error" + ex );
			String title = res.getString( "connector.err3.title" );
			String msg = res.getString( "connector.err3.message" );
			if ( ex.getMessage() == null ) {
				msg += "\n" + ex.toString();
			} else {
				msg += "\n" + ex.getMessage();
			}
			JOptionPane.showMessageDialog( this,
					msg, title, JOptionPane.ERROR_MESSAGE );
			return;
		}
		bOk = true;
		setVisible( false );
	}

	public void onCancel() {
		setVisible( false );
	}

	public boolean isOk() {
		return bOk;
	}

	public void setEnabled( boolean bEn ) {
		if ( !bEn ) {
			BTN_CANCEL.requestFocus();
		}
		super.setEnabled( bEn );
		BTN_OK.setEnabled( bEn );
		if ( connectorPanel != null ) {
			connectorPanel.setEnabled( bEn );
		}
	}

}
