package jp.co.sds_corp.hibiya.monitor;

import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.MouseListener;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;

import javax.swing.Icon;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import jp.co.sds_corp.hibiya.connector.IConnector;
import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;

/**
* 指定されたグループに所属する{@link IChecker}の一覧を表示します。
*/
public class ListTable extends JTable
{
	private static final long serialVersionUID = 1L;
	JScrollPane		sp;
	SetupParam		setup;
	MonitorParam	prm;

	public ListTable( SetupParam setup, MonitorParam prm ) {
		this.setup = setup;
		setParam( prm );
	}

	protected void setParam( MonitorParam prm ) {
		this.prm = prm;

		ListModel model = new ListModel( prm );
		setModel( model );
		int height = setup.getRowHeight();
		if ( height < 30 ) {
			height = 30;
		}
		setRowHeight( height );
		setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		setAutoResizeMode( JTable.AUTO_RESIZE_NEXT_COLUMN );
		setColumnSelectionAllowed( false );

		for ( int nLoop = 0; nLoop < prm.getColumnCount(); nLoop++ ) {
			TableColumn column = null;
			column = getColumnModel().getColumn( nLoop );
			column.setWidth( prm.getColumnWidth( nLoop ) );
			column.setPreferredWidth( prm.getColumnWidth( nLoop ) );
		}
		setIconColumn( null );
		sp = new JScrollPane( this );
	}

	protected void setIconColumn( Component c ) {
		if ( prm == null ) {
			return;
		}
		int nPos = prm.getColumnPos( "Status" );
		if ( nPos >= 0 ) {
			TableColumn col = getColumnModel().getColumn( nPos );
			col.setCellRenderer( new StatCellRenderer() );
			int height = calcRowHeight( c ) + 5;
			col.setMaxWidth( height );
			col.setMinWidth( height );
			col.setPreferredWidth( height );
		}
	}

	public void addMouseListener2( MouseListener l ) {
		super.addMouseListener( l );
		if ( sp != null ) {
			sp.addMouseListener( l );
		}
	}

	/**
	* ICheckerの状態に変更があった時に呼び出されます。
	* @param	ev		状態の変更内容を指定します。
	*/
	public void checkerChanged( CheckerChangeEvent ev ) throws Exception {
		IChecker mon = (IChecker )ev.getSource();
		if ( ev.getNewStatus() == IChecker.Status.DEL ) {
			((ListModel )getModel()).remove( mon );
		} else {
			((ListModel )getModel()).setData( mon );
		}
	}

	public void saveColumn() {
		ListModel model = (ListModel )getModel();
		for ( int i = 0; i < model.getColumnCount(); i++ ) {
			int var;
			var = getColumnModel().getColumn(i).getWidth();
			prm.setColumnWidth( i, var );
			var = convertColumnIndexToModel( i );
			prm.setOrder( var, i );
		}
	}

	public MonitorParam getParam() {
		return prm;
	}

	public JScrollPane getScrollPane() {
		return sp;
	}

	public void setFont( Font font, Component c ) {
		super.setFont( font );
		getTableHeader().setFont( font );
		int height = calcRowHeight( c );
		setRowHeight( height );
		setIconColumn( c );
	}

	private int calcRowHeight( Component c ) {
		int ret = getRowHeight();
		Font font = getFont();
		if ( c == null ) {
			return ret;
		}
		Graphics2D grf = (Graphics2D )c.getGraphics();
		if ( grf == null ) {
			return ret;
		}
		FontRenderContext frc = grf.getFontRenderContext();
		Rectangle2D rect = font.getMaxCharBounds( frc );
		ret = (int )rect.getHeight();
		ret = (int )font.getLineMetrics( "Aあ", frc ).getHeight();
		if ( ret < 30 ) {
			ret = 30;
		}
		return ret;
	}

	class StatCellRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent( JTable tbl, Object obj,
					boolean isSelected, boolean hasFocus, int row, int clm )
		{
			ListModel model = (ListModel )tbl.getModel();
			IChecker chk = (IChecker )model.getData( row );
			IConnector con = model.getConnector();
			IChecker.Status stat= chk.getCheckerStatus();
			if ( !con.isConnected()) {
				stat = IChecker.Status.INIT;
			}
			Icon i;
			int height = calcRowHeight( this );
			i = prm.getSizedIcon( stat, height );
			setIcon( i );
			return this;
		}
	}

}

