package jp.co.sds_corp.hibiya.monitor;

import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.text.Position;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import jp.co.sds_corp.hibiya.connector.IConnector;
import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.util.common.CmnLog;

/**
* {@link IChecker}のツリーを表示します。
*/
public class MonitorTree extends JTree implements ICheckerListener
{
	private static final long serialVersionUID = 1L;
	SetupParam	setup;
	MonitorTreeModel	model = null;
	DefaultMutableTreeNode		topNode;
	JLabel tmp = new JLabel();

	public MonitorTree( SetupParam setup ) {
		this.setup = setup;
		setShowsRootHandles( false );
		int height = setup.getRowHeight();
		setRowHeight( height );
		MonitorTreeCellRenderer render = new MonitorTreeCellRenderer();
		setCellRenderer( render );

		NodeInfo info;
		info = new NodeInfo( NodeInfo.Level.ROOT );
		topNode = new DefaultMutableTreeNode( info );
		model = new MonitorTreeModel( topNode );
		setModel( model );
	}

	public TreePath getNextMatch( String prefix, int startingRow,
		Position.Bias bias )
	{
		try {
			return super.getNextMatch( prefix, startingRow, bias );
		} catch ( Exception ex ) {
			return null;
		}
	}

	public void addConnector( IConnector con, Map<String,MonitorParam> mapPrm ) {
		NodeInfo info;
		info = new NodeInfo( NodeInfo.Level.CONNECTOR, con );

		DefaultMutableTreeNode nodeCon;
		nodeCon = new DefaultMutableTreeNode( info );
		topNode.add( nodeCon );
		model.reload();
	}

	public void updateChecker( IConnector con, Map<String,MonitorParam> mapPrm )
		throws Exception
	{
		DefaultMutableTreeNode nodeCon =
			(DefaultMutableTreeNode )findConnectorNode( con );

		Iterator<String> itr = mapPrm.keySet().iterator();
		while ( itr.hasNext()) {
			MonitorParam prm = (MonitorParam )mapPrm.get( itr.next());
			addNode( nodeCon, con, prm );
		}
		Map<String,IChecker> map = con.getCheckerMap();
		itr = map.keySet().iterator();
		while ( itr.hasNext()) {
			Object key = itr.next();
			IChecker chk = (IChecker )map.get( key );
			MonitorParam prm = findParam( mapPrm, chk );
			if ( prm == null ) {
				CmnLog.error( this, "can't find parameter for " + chk );
				continue;
			}
			DefaultMutableTreeNode nodeGrp =
				(DefaultMutableTreeNode )findGroupNode( con, prm );
			if ( chk.getCheckerStatus() != IChecker.Status.DEL ) {
				addChecker( nodeGrp, con, prm, chk );
			}
		}
		if ( !con.isDisplayOnBlank()) {
			for ( int i = nodeCon.getChildCount() - 1; i >= 0; i-- ) {
				DefaultMutableTreeNode grp =
					(DefaultMutableTreeNode )nodeCon.getChildAt( i );
				if ( grp.getChildCount() == 0 ) {
					grp.removeFromParent();
				}
			}
		}
		model.reload();
	}

	public void updateChecker( IConnector con, MonitorParam prm )
	{
		DefaultMutableTreeNode nodeGrp =
			(DefaultMutableTreeNode )findGroupNode( con, prm );
		nodeGrp.removeAllChildren();

		Map<String,IChecker> map = null;
		try {
			map = con.getCheckerMap();
		} catch ( Exception ex ) {
			CmnLog.error( this, "getCheckerMap() error", ex );
			return;
		}
		Iterator<String> itr = map.keySet().iterator();
		while ( itr.hasNext()) {
			Object key = itr.next();
			IChecker chk = (IChecker )map.get( key );
			if ( !prm.isValidChecker( chk )) {
				continue;
			}
			if ( chk.getCheckerStatus() != IChecker.Status.DEL ) {
				addChecker( nodeGrp, con, prm, chk );
			}
		}
		if ( !con.isDisplayOnBlank()) {
			if ( nodeGrp.getChildCount() == 0 ) {
				nodeGrp.removeFromParent();
			}
		}
		model.reload();
		expandAll();
	}

	private MonitorParam findParam( Map<String,MonitorParam> mapPrm, IChecker chk ) {
		Iterator<String> itr = mapPrm.keySet().iterator();
		while ( itr.hasNext()) {
			MonitorParam prm = (MonitorParam )mapPrm.get( itr.next());
			if ( prm.isValidChecker( chk )) {
				return prm;
			}
		}
		return null;
	}

	private DefaultMutableTreeNode addNode( DefaultMutableTreeNode node,
			IConnector con, MonitorParam prm )
	{
		NodeInfo info;
		info = new NodeInfo( NodeInfo.Level.GROUP, con, prm );
		DefaultMutableTreeNode child;
		child = new DefaultMutableTreeNode( info );
		((DefaultMutableTreeNode )node).add( child );
		return child;
	}

/*
	public void addChecker( IConnector con, MonitorParam prm, IChecker chk ) {
		DefaultMutableTreeNode node =
			(DefaultMutableTreeNode )findGroupNode( con, prm );
		if ( node == null ) {
			CmnLog.error( this, "node not found." );
			return;
		}
		NodeInfo info = new NodeInfo( NodeInfo.Level.CHECKER, con, prm, chk );
		DefaultMutableTreeNode leaf;
		leaf = new DefaultMutableTreeNode( info );
		model.insertNodeInto( leaf, node, 0 );
	}
*/

	private DefaultMutableTreeNode addChecker( DefaultMutableTreeNode node,
			IConnector con, MonitorParam prm, IChecker chk )
	{
		NodeInfo info = new NodeInfo( NodeInfo.Level.CHECKER, con, prm, chk );
		DefaultMutableTreeNode leaf;
		leaf = new DefaultMutableTreeNode( info );
		node.add( leaf );
		return leaf;
	}

	/**
	* ICheckerの状態に変更があった時に呼び出されます。
	* @param	ev		状態の変更内容を指定します。
	*/
	public void checkerChanged( CheckerChangeEvent ev ) throws Exception {
		IChecker chk = (IChecker )ev.getSource();
		DefaultMutableTreeNode node =
			(DefaultMutableTreeNode )findCheckerNode( topNode, ev.getConnector(), chk );
		if ( node != null ) {
			if ( ev.getNewStatus() == IChecker.Status.DEL ) {
				model.removeNodeFromParent( node );
			} else {
				NodeInfo info = (NodeInfo )node.getUserObject();
				info.setChecker( chk );
				model.nodeChanged( node );
			}
		}
	}

	protected TreeNode findConnectorNode( IConnector con ) {
		for ( int i = 0; i < topNode.getChildCount(); i++ ) {
			DefaultMutableTreeNode child =
				(DefaultMutableTreeNode )topNode.getChildAt( i );
			NodeInfo info = (NodeInfo )child.getUserObject();
			if ( info.getLevel() != NodeInfo.Level.CONNECTOR ) {
				continue;
			}
			if ( con.equals( info.getConnector())) {
				return child;
			}
		}
		return null;
	}

	protected TreeNode findGroupNode( IConnector con, MonitorParam prm ) {
		TreeNode node = findConnectorNode( con );
		for ( int i = 0; i < node.getChildCount(); i++ ) {
			DefaultMutableTreeNode child =
				(DefaultMutableTreeNode )node.getChildAt( i );
			NodeInfo info = (NodeInfo )child.getUserObject();
			if ( info.getLevel() != NodeInfo.Level.GROUP ) {
				continue;
			}
			if ( con.equals( info.getConnector()) &&
				info.getParam().getName().equals( prm.getName() ))
			{
				return child;
			}
		}
		return null;
	}

	protected TreeNode findCheckerNode( TreeNode node,
			IConnector con, IChecker chk )
	{
		for ( int i = 0; i < node.getChildCount(); i++ ) {
			DefaultMutableTreeNode child =
				(DefaultMutableTreeNode )node.getChildAt( i );
			if ( child.getChildCount() > 0 ) {
				TreeNode found = findCheckerNode( child, con, chk );
				if ( found != null ) {
					return found;
				}
			}
			NodeInfo info = (NodeInfo )child.getUserObject();
			if ( info.getLevel() != NodeInfo.Level.CHECKER ) {
				continue;
			}
			if ( chk.getHashCode() == info.getChecker().getHashCode() ) {
				if ( con == null ) {
					return child;
				}
				if ( con.equals( info.getConnector()) ) {
					return child;
				}
			}
		}
		return null;
	}
/*
	public boolean isEditable() {
		return true;
	}
*/

	public void updateConnector( IConnector con, Map<String,MonitorParam> mapPrm )
		throws Exception
	{
		DefaultMutableTreeNode conNode =
			(DefaultMutableTreeNode )findConnectorNode( con );
		if ( conNode != null ) {
			conNode.removeAllChildren();
		}
		updateChecker( con, mapPrm );
		model.reload();
		expandAll();
	}

	public void removeConnector( IConnector con ) {
		TreeNode node = findConnectorNode( con );
		if ( node != null ) {
			model.removeNodeFromParent( (MutableTreeNode )node );
		}
	}

	public void removeChecker( IConnector con, IChecker chk ) {
		TreeNode node = findCheckerNode( topNode, con, chk );
		if ( node != null ) {
			model.removeNodeFromParent( (MutableTreeNode )node );
		}
	}

	public void removeLeaf( MutableTreeNode node ) {
		model.removeNodeFromParent( node );
	}

	public void removeAll() {
		TreePath[] all = getPathBetweenRows( 0, getRowCount() );
		removeSelectionPaths( all );
	}

	public void expand( DefaultMutableTreeNode node ) {
		TreePath path = new TreePath( node.getPath() );
		expandPath( path );
	}

	public void expandAll( DefaultMutableTreeNode node ) {
		for ( int i = 0; i < node.getChildCount(); i++ ) {
			DefaultMutableTreeNode child =
				(DefaultMutableTreeNode )node.getChildAt( i );
			expandAll( child );
		}
		expand( node );
	}

	public void expandAll() {
		for ( int i = 0; i < getRowCount(); i++ ) {
			expandRow( i );
		}
	}

	public void setFont( Font font ) {
		super.setFont( font );
		int height = calcRowHeight();
		setRowHeight( height );
	}

	private int calcRowHeight() {
		int ret = getRowHeight();
		Font font = getFont();
		Graphics2D grf = (Graphics2D )getGraphics();
		if ( grf == null ) {
			return ret;
		}
		FontRenderContext frc = grf.getFontRenderContext();
		Rectangle2D rect = font.getMaxCharBounds( frc );
		ret = (int )rect.getHeight();
		ret = (int )font.getLineMetrics( "Aあ", frc ).getHeight();
		return ret;
	}

	class MonitorTreeModel extends DefaultTreeModel
	{
		private static final long serialVersionUID = 1L;

		MonitorTreeModel( TreeNode node ) {
			super( node );
		}
		public boolean isLeaf( Object obj ) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode )obj;
			NodeInfo info = (NodeInfo )node.getUserObject();
			if ( info.getLevel().compareTo( NodeInfo.Level.CHECKER ) < 0 ) {
				return false;
			}
			return true;
		}
		public void removeNodeFromParent(MutableTreeNode node) {
			super.removeNodeFromParent(node);
		}
	}

	class MonitorTreeCellRenderer extends DefaultTreeCellRenderer {
		private static final long serialVersionUID = 1L;

		public Component getTreeCellRendererComponent(
							JTree tree,
							Object value,
							boolean sel,
							boolean expanded,
							boolean leaf,
							int row,
							boolean hasFocus)
		{
			Component c = super.getTreeCellRendererComponent(
				tree, value, sel, expanded, leaf, row, hasFocus );
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
			if ( ! ( node.getUserObject() instanceof NodeInfo )) {
				return c;
			}
			NodeInfo info = (NodeInfo)node.getUserObject();
			if ( info == null ) {
				return c;
			}
			switch ( info.getLevel() ) {
			case CONNECTOR:
				c = getConnectorComponent( c, info );
				break;
			case CHECKER:
				c = getCheckerComponent( c, info );
				break;
			default:
			}
			return c;
		}

		private Component getConnectorComponent( Component c, NodeInfo info ) {
			IConnector con = info.getConnector();
			IChecker.Status stat = ( con.isConnected() ?
				IChecker.Status.OK : IChecker.Status.NG );
			int height = calcRowHeight();
			Icon icon = setup.getSizedIcon( stat, height );
			((JLabel )c).setText( con.getName() );
			((JLabel )c).setIcon( icon );
			return c;
		}

		private Component getCheckerComponent( Component c, NodeInfo info ) {
			IChecker.Status stat;
			IChecker chk = info.getChecker();
			if ( chk == null ) {
				stat = IChecker.Status.INIT;
			} else {
				stat = chk.getCheckerStatus();
				((JLabel )c).setText( chk.getName() );
			}
			IConnector con = info.getConnector();
			if ( !con.isConnected()) {
				stat = IChecker.Status.INIT;
			}

			MonitorParam prm = info.getParam();
			int height = calcRowHeight();
			Icon icon = setup.getSizedIcon( stat, height );
			((JLabel )c).setIcon( icon );
			return c;
		}
	}

}
